/*
   FALCON - The Falcon Programming Language.
   FILE: apache_output.cpp

   Falcon module for Apache 2

   Apache aware error handler.
   -------------------------------------------------------------------
   Author: Giancarlo Niccolai
   Begin: Wed, 30 Apr 2008 18:18:28 +0200

   -------------------------------------------------------------------
   (C) Copyright 2008: the FALCON developers (see list in AUTHORS file)

 */

#include "apache_output.h"
#include <falcon/autocstring.h>

#include <util_filter.h>
#include <http_protocol.h>
#include "apache_vm.h"

ApacheOutput::ApacheOutput( request_rec *rec ):
   m_request( rec ),
   m_bHeaderSent( false ),
   m_bClosed( false ),
   m_lastError( 0 ),
   m_vm( 0 )
{
   // create the bucket brigade we may need
   m_brigade = apr_brigade_create(rec->pool, rec->connection->bucket_alloc);
}

void ApacheOutput::sendHeaders()
{
   s_sendHeaders( m_brigade, this );
}

bool ApacheOutput::write( const char *data, int size )
{
   if ( m_bClosed )
      return false;

   m_lastError =  apr_brigade_write( m_brigade,
      s_sendHeaders,
      this,
      data,
      size
   );

   return m_lastError == 0;
}

bool ApacheOutput::write( const Falcon::String &data )
{
   if ( m_bClosed )
     return false;

   // todo: manage encoding
   Falcon::AutoCString cstr( data );

   m_lastError =  apr_brigade_write( m_brigade,
      s_sendHeaders,
      this,
      cstr.c_str(),
      cstr.length()
   );

   return m_lastError == 0;

}

void ApacheOutput::flush()
{
   if ( !m_bClosed )
   {
      if ( ! m_bHeaderSent )
         sendHeaders();

      ap_filter_flush( m_brigade, m_request->output_filters);
      //ap_pass_brigade( m_request->output_filters, m_brigade );
      ap_rflush( m_request );

   }
}

void ApacheOutput::close()
{
   flush();
   apr_brigade_cleanup( m_brigade );
   m_bClosed = true;
}

apr_status_t ApacheOutput::s_sendHeaders( apr_bucket_brigade *bb, void *ctx )
{
   ApacheOutput *aout = (ApacheOutput*) ctx;

   if ( ! aout->m_bHeaderSent  )
   {
      // In case we can't have the VM to setup our reply...
      if ( aout->m_vm == 0 || ! aout->m_vm->prepareHeader() )
      {
         // prepare a default.
         apr_table_t *h = aout->request()->headers_out;
         apr_table_set( h, "Content-type", "text/html;charset=utf-8" );
         apr_table_set( h, "Pragma", "no-cache");
         apr_table_set( h, "Cache-Control", "no-cache");
         aout->request()->content_type = "text/html;charset=utf-8";
      }
      
      ap_rflush( aout->request() );
      aout->m_bHeaderSent = true;
   }
   
   // perform real flushing
   ap_filter_flush( aout->m_brigade, aout->request()->output_filters );

   return APR_SUCCESS;
}

void ApacheOutput::setVM( ApacheVM *vm )
{
   m_vm = vm;
   if( m_bHeaderSent )
      m_vm->headerSent();
}
