/*
   FALCON - The Falcon Programming Language.
   FILE: apache_output.h

   Falcon module for Apache 2

   Little utilities to write data on the apache logical memory
   output stream.
   -------------------------------------------------------------------
   Author: Giancarlo Niccolai
   Begin: Wed, 30 Apr 2008 18:18:28 +0200

   -------------------------------------------------------------------
   (C) Copyright 2008: the FALCON developers (see list in AUTHORS file)

 */

#ifndef MOD_FALCON_OUTPUT_DATA
#define MOD_FALCON_OUTPUT_DATA

#include "mod_falcon.h"
#include <falcon/string.h>
#include <apr_buckets.h>

class ApacheVM;

class ApacheOutput
{
   request_rec *m_request;
   apr_bucket_brigade *m_brigade;
   bool m_bHeaderSent;
   bool m_bClosed;
   apr_status_t m_lastError;
   ApacheVM *m_vm;

   // This is for future expansion
   //const char *m_sEncoding;

public:

   ApacheOutput( request_rec *rec );
   ~ApacheOutput()
   {
      close();
   }

   bool headerSent() const { return m_bHeaderSent; }
   void sendHeaders();

   static apr_status_t s_sendHeaders( apr_bucket_brigade *bb, void *ctx );
   bool write( const char *data, int size );
   bool write( const Falcon::String &data );
   void flush();
   void close();
   apr_status_t lastError() const { return m_lastError; }
   request_rec *request() const { return m_request; }

   /** Sets the VM instance working with this output.
      The vm instance is used to retreive the headers when it's time to
      write them. As the Output may work independently from the VM,
      and be used i.e. to send Apache an error document before the
      VM is readied, this method allows for later introduction of the VM.
   */
   void setVM( ApacheVM *vm );
};

#endif
