/*
   FALCON - The Falcon Programming Language.
   FILE: apache_vm.h

   Falcon module for Apache 2
   Apache specific Falcon VM.
   -------------------------------------------------------------------
   Author: Giancarlo Niccolai
   Begin: 2007-07-28 18:55:17

   -------------------------------------------------------------------
   (C) Copyright 2008: the FALCON developers (see list in AUTHORS file)

   See LICENSE file for licensing details.
*/

#ifndef APACHE_VM_H
#define APACHE_VM_H

#include <falcon/vm.h>
#include <falcon/coredict.h>
#include "mod_falcon.h"
#include <apr.h>

class ApacheOutput;


class ApacheVM: public Falcon::VMachine
{
   // The request that this VM must handle
   request_rec *m_request;
   char reply_buffer[256];

   // we keep the output just for reference.
   ApacheOutput *m_output;

   // when the script begun being processed.
   Falcon::numeric m_startedAt;

   class TempFileEntry
   {
   public:
      Falcon::String m_entry;
      TempFileEntry* m_next;

      TempFileEntry( const Falcon::String &fname );
   };

   // Used to remember which files to delete at end.
   TempFileEntry *m_tempFiles;
   
public:
   ApacheVM( request_rec *request, ApacheOutput *out, bool init = true );
   virtual ~ApacheVM();

   /** Parses the query string and sets it in a dictionary. */
   Falcon::CoreDict *parseQuery( const Falcon::String &query );

   /** Tells the "Header" well known item that headers are gone.*/
   void headerSent();

   /** Reads the settings in the "Header" well know item and prepare the reply */
   bool prepareHeader();

   request_rec  *request() const { return m_request;}
   ApacheOutput *apacheOutput() const { return m_output; }

   void startedAt( Falcon::numeric t ) { m_startedAt = t; }
   Falcon::numeric startedAt() const { return m_startedAt; }

   /** Adds a temporary file.
      The VM tries to delete all the temporary files during its destructor.
      On failure, it ingores the problem and logs an error in Apache.
   */
   void addTempFile( const Falcon::String &fname );

   /** Gets the list of temporary files.
      Before the VM is destroyed, this should be taken out
      so that it is then possible to get rid of the files.

      Using removeTempFiles after the VM has been destroyed ensures
      that all the streams open by the VM are closed (as this is done
      during the GC step).

      \return an opaque pointer to an internal structure.
   */
   void* getTempFiles() const { return m_tempFiles; }

   /** Removes from the disk a list of temporary files.

      Using removeTempFiles after the VM has been destroyed ensures
      that all the streams open by the VM are closed (as this is done
      during the GC step).
      
      \param p a pool used for logging in case of problems.
      \param head The valued returned from getTempFiles() before the VM was destroyed.
   */
   static void removeTempFiles( apr_pool_t* p, void* head );

   void showrefs();
};

#endif

/* end of apache_vm.h */

